# -*- coding: utf-8 -*-
"""
Created on Sat Sep  2 20:32:27 2017

@author: Yuki
"""
import sys,threading

from abc import abstractmethod
import multiprocessing
from multiprocessing import Process,Queue

from PyQt5.QtWidgets import QApplication,QMainWindow,QDockWidget,QDesktopWidget
from PyQt5.QtCore import pyqtSignal,Qt,QThread,QTimer
from PyQt5.QtGui import QIcon
import pyqtgraph as pg

import kuchinawa

class AddSignal:
    '''
    A class which capsule information to instantiate a graph in another process.
    '''
    def __init__(self,identifier,graph,name,*args,**kwargs):
        self.identifier=identifier
        self.graph=graph
        self.name=name
        self.args=args
        self.kwargs=kwargs
        
class RemoveSignal:
    '''
    A class which capsule information to remove a graph in another process.
    '''
    def __init__(self,identifier):
        self.identifier=identifier
        
class DataSignal:
    '''
    A class which capsule data to plot onto a graph in another process.
    '''
    def __init__(self,identifier,data):
        self.identifier=identifier
        self.data=data

class ResetSignal:
    '''
    The instance of this class resets GraphContainer.
    '''
    pass

class PoisonPill:
    '''
    The instance of this class kills GraphContainer.
    '''
    pass
        
class GraphInterface:
    '''
    A interface to plot data onto a graph in another process. An instance of this class holds Queue connected to GraphContainer.
    '''
    Total=0
    def __init__(self,que,graph,name,*args,**kwargs):
        self.que=que
        self.identifier=GraphInterface.Total
        self.que.put(AddSignal(self.identifier,graph,name,*args,**kwargs))
        GraphInterface.Total+=1
        
    def put(self,data):
        '''
        Put data to the Queue connected to the graph
        
        Parameters:
            data: object
                Any object can be given. It depends on the implementation of the graph class how the data is treated
        '''
        identified_data=DataSignal(self.identifier,data)
        self.que.put(identified_data)
        
    def remove(self):
        self.que.put(RemoveSignal(self.identifier))
        
class watchQue(QThread):
    addSignal=pyqtSignal(int,object,str,tuple,dict)
    dataSignal=pyqtSignal(int,object)
    resetSignal=pyqtSignal()
    removeSignal=pyqtSignal(int)
    poisonSignal=pyqtSignal()
    
    def __init__(self,que,gc):
        super().__init__()
        self.que=que 
        self.gc=gc #GraphContainer
        self.stop_event=threading.Event()
        
    def run(self):
        while not self.stop_event.is_set():
            signal=self.que.get()
            if isinstance(signal,DataSignal):
                self.dataSignal.emit(signal.identifier,signal.data)
            elif isinstance(signal,AddSignal):
                self.addSignal.emit(signal.identifier,signal.graph,signal.name,signal.args,signal.kwargs)
            elif isinstance(signal,RemoveSignal):
                self.removeSignal.emit(signal.identifier)
            elif isinstance(signal,ResetSignal):
                self.resetSignal.emit()
            elif isinstance(signal,PoisonPill):
                self.poisonSignal.emit()
                self.stop()

    def stop(self):
        self.stop_event.set()
        
class DockList:
    '''
    A list of QDockWidgets holds a index. The QDockWidgets at the index or later have no child.
    '''
    def __init__(self):
        self.docks=[]
        self.index=None #None means all the QDockWidgets have a child
    
    def setWidget(self,widget,name):
        '''
        Set a widget to the QDockWidget which has no child at the minimum index.
        
        Parameters:
            widget: pg.PlotWidget
                a widget to add
            name: str
                the window title of the QDockWidget
        Returns:
            (bool,QDockWidget)
                bool: True if generated a new QDockWidget
                QDockWidget: the QDockWidget to which the widget was added
        '''
        if self.index==None:
            dock=QDockWidget()
            self.docks.append(dock)
            dock.setWidget(widget)
            dock.setWindowTitle(name)
            return (True,dock)
        else:
            dock=self.docks[self.index]
            if self.index==len(self.docks)-1:
                self.index=None
            else:
                self.index+=1
            dock.setWidget(widget)
            dock.setWindowTitle(name)
            return (False,dock)
            
    def resetAll(self):
        '''
        Reset the window titles of All the QDockWidgets in self.docks and set self.index=0
        
        Returns:
            None
        '''
        if len(self.docks)==0:
            self.index=None
        else:
            for q in self.docks:
                q.setWindowTitle('')
                q.show()
            self.index=0
        
    def getEmpties(self):
        '''
        Return all the QDockWidgets which have no child in a list.
        
        Returns:
            list
        '''
        if self.index==None:
            answer=[]
        else:
            answer=self.docks[self.index:]
        return answer
        
class GraphContainer(QMainWindow):
    '''
    A container class which holds graphs
    '''
    DOCKOPTIONS=QMainWindow.AllowTabbedDocks|QMainWindow.AllowNestedDocks
    
    def __init__(self,que):
        super().__init__()
        self.setDockOptions(GraphContainer.DOCKOPTIONS)
        self.graphs={} #dict to manage graphs: {int: (Queue, GraphBase) }
        self.docks=DockList()
        self.que=que #the connection to the main process
        
        self.watchThread=watchQue(self.que,self)
        self.watchThread.addSignal.connect(self.addGraph)
        self.watchThread.dataSignal.connect(self.putData)
#        self.watchThread.removeSignal.connect(self.removeGraph)
        self.watchThread.resetSignal.connect(self.reset)
        self.watchThread.poisonSignal.connect(self.close)
        self.watchThread.start()
        
        #place the window to the right of the screen
        screen=QDesktopWidget().availableGeometry()
        height=screen.height()
        width=screen.width()/2
        self.setGeometry(width,40,width,height)
        
        #self.setWindowFlags(self.windowFlags() & ~Qt.WindowCloseButtonHint)
        self.setWindowIcon(QIcon(kuchinawa.ICONPATH))
        self.setWindowTitle(kuchinawa.KUCHINAWA+' Graph Container')
        self.show()

    
    def addGraph(self,identifier,graph,name,args,kwargs):
        '''
        Embed 'graph' to QDockWidget and add it to self
        
        Paramters:
            identifier: int
                identifier to distinguish between graphs
            graph: class
                the class to generate
            args: tuple
                positional arguments to give to graph's __init__
            kwargs: dict
                keyword arguments to give to graph's __init__
        Returns:
            None
        '''
        g=graph(*args,**kwargs)
        mybool,dock=self.docks.setWidget(g,name)
        que=g.getQue()
        self.graphs[identifier]=(que,g)
        if mybool:
            self.addDockWidget(Qt.TopDockWidgetArea,dock)
        
    def putData(self,identifier,data):
        self.graphs[identifier][0].put(data) #self.graphs holds {int: (Queue, GraphBase)}
        
#    def removeGraph(self,identifier):
#        '''
#        Remove the graph identified by 'identifier'
#        
#        Parameters:
#            identifier: int
#                the key in self.graphs
#        Returns:
#            None
#        '''
#        mytuple=self.graphs[identifier]
#        que=mytuple[0]
#        graph=mytuple[1]
#        graph.setParent(None)
#        del(que)
#        del(graph)
        
    def reset(self):
        for mytuple in self.graphs.values(): #self.graphs holds {int: (Queue, GraphBase)}
            que=mytuple[0]
            graph=mytuple[1]
            graph.setParent(None)
            del(que)
            del(graph)
        self.docks.resetAll()
    
class GraphBase(pg.PlotWidget):
    '''
    A Base class which can be added to GraphContainer
    '''
    GRAPH_RATE=100  #update rate (/millisecond)
    
    def __init__(self):
        super().__init__()
        
        self.que=Queue()
        
        self.timer=QTimer()
        self.timer.timeout.connect(self.update)
        
        self.plt=self.plotItem
        self.plt.showGrid(x=True,y=True)
        fontCss = {'font-family': "Segoe UI, メイリオ", 'font-size': '14pt', "color": 'white'}
        self.plt.getAxis('bottom').setLabel(**fontCss)
        self.plt.getAxis('left').setLabel(**fontCss)
        
    def getQue(self):
        return self.que
        
    @abstractmethod
    def update(self):
        '''Implementation of graph update'''
        raise NotImplementedError('This method must be implemented in a subclass.')
        
    def start(self):
        self.timer.start(self.GRAPH_RATE)
        
class ScatterAll(GraphBase):
    '''
    Plot all data and highlight the latest point. Put a new data in a dict: {'x': x value,'y': y value}
    '''
    def __init__(self,xlabel,xunit,ylabel,yunit,color='w'):
        super().__init__()
        #Initialization of PlotItem
        self.xlabel=xlabel
        self.ylabel=ylabel
        
        self.x=[]
        self.y=[]

        self.point=None

        self.plt.setLabel('bottom',xlabel, units=xunit)
        self.plt.setLabel('left', ylabel, units=yunit)
        
        self.curve=self.plt.plot(self.x,self.y,pen=color)
        self.start()
        #set markers at latest values if data lenght is not zero.
        
    def update(self):
        mylist=self.empty_que()
        if len(mylist)>0:
            xappended=[d['x'] for d in mylist]
            self.x=self.x+xappended
            self.x_last=xappended[-1]
            yappended=[d['y'] for d in mylist]
            self.y=self.y+yappended
            self.y_last=yappended[-1]

            if not self.point==None:
                self.plt.removeItem(self.point)
                
            self.curve.setData(self.x,self.y)
            self.point=self.plt.plot([self.x_last],[self.y_last],symbolBrush='w')
        
    def empty_que(self):
        '''Take out data from the que and return them in a list.'''
        mylist=[]
        while True:
            if not self.que.empty():
                v=self.que.get()
                mylist.append(v)
            else:
                break
        return mylist
        
def initGraphContainer(que):
    app = QApplication(sys.argv)
    gc=GraphContainer(que)
    app.exec_()


# -*- coding: utf-8 -*-
"""
Created on Sat Sep  2 13:41:09 2017

@author: Yuki
"""
from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QColor

from kuchinawa import Main,Graph
import sys,time
from datetime import datetime
import numpy as np

#QtDesignerで作成した拡張子が.uiのファイルをコンパイルした拡張子が.pyのファイルをimportする　拡張子は含めない
#import the file with the extension .py which was compiled from a file with extension .ui made by using QtDesigner. not include the extension.
from kuchinawa.Examples import SinCos_ui

COLORS=[QColor(255,0,0),QColor(255,255,0),QColor(0,255,0)] #the colors of points in graphs
#declaration of constants
T='time'
Sin='sin'
Cos='cos'



class Sample(Main):
    def __init__(self):
        super().__init__()
        self.ui=SinCos_ui.Ui_Form() #get a user interface made by using QtDesigner
        self.setUI(self.ui) #set the user interface
        
    def run(self):
        #ファイルダイアログを出して指定された場所にcsvファイルを作成し、そのファイルへアクセスできるインスタンスを取得する
        #get a file instance connected to the csv file which is created to the path appointed in a file dialog.
        file=self.getSaveFile()
        
        #ユーザインタフェース上のウィジットが持つ関数を呼び出す場合は以下のようにself.callを使う。これはスレッドセーフな方法でウィジットを使うために必要。
        #第一引数は関数名、第二、三引数はそれぞれタプル、辞書型でポジショナル、キーワード引数として関数に渡される
        #Please use 'self.call' to call a function of a widget in the user interface. This method provides a thread-safe way 
        #to interact with widgets. The first argument is the name of the function. The second and third argument are 
        #a tuple and dict to be handed to the function as positional and keyword argument respectively.
        self.call(self.ui.lineEdit.setText,(file.path,))
        file.write_header([T,Sin,Cos]) #write headers to the file
        
        #Generate graphs by using 'self.addGraph' and get graph instances. The first argument is the graph class to instantiate. 
        #This class must inherit kuchinawa.Graph.GraphBase. The second and third argument are 
        #a tuple and dict to be handed to the function as positional and keyword argument respectively.
        #The fourth argument is the label of the graph in the GraphContainer.
        g_sin=self.addGraph(Graph.ScatterAll,(T,'sec',Sin,'V'),{'color':COLORS[0]},'Sin')
        g_cos=self.addGraph(Graph.ScatterAll,(T,'sec',Cos,'V'),{'color':COLORS[1]},'Cos')
        g_orbit=self.addGraph(Graph.ScatterAll,(Cos,'V',Sin,'V'),{'color':COLORS[2]},'Orbit')
        timeOrigin=datetime.now() #get the origin of time
        while self.running==True:
            t=(datetime.now()-timeOrigin).total_seconds() #get an elapsed time from the origin of time
            
            #振幅、位相、ノイズをユーザインタフェース上のウィジットから取得して時間に対するsin,cosを計算する
            #Calculate a sin and cos wave function of time. Get the amplitude, phase and noise level from widgets on the user interface.
            sin=self.call(self.ui.amp_sin.value)*np.sin(t+np.pi*self.call(self.ui.dial_sin.value)/180)+\
                self.call(self.ui.noise_sin.value)*np.random.rand() # 
            cos=self.call(self.ui.amp_cos.value)*np.cos(t+np.pi*self.call(self.ui.dial_cos.value)/180)+\
                self.call(self.ui.noise_cos.value)*np.random.rand()
            #display the calculated values to the LCD displays on the user interface
            self.call(self.ui.lcdNumber_sin.display,(sin,))
            self.call(self.ui.lcdNumber_cos.display,(cos,))
            #グラフインスタンスを介してグラフにデータをプロットする。どういう形式でデータを送るか（putに何を渡すか）はグラフクラスの実装に依存するがkuchinawa.Graph.ScatterAllの
            #場合はdictで送信。　キー'x'の値がx座標、'y'の値がy座標
            #Plot values to graphs　via graph instances. It depends on the implementation of a graph class how to send data to the graph, that is, 
            #what you hand to 'kuchinawa.Graph.GraphInterface.put' method. In case of kuchinawa.Graph.ScatterAll, users send data 
            #represented as a dict. The values of key 'x' and 'y' are the x and y coordinate of a data point respectively.
            g_sin.put({'x':t,'y':sin})
            g_cos.put({'x':t,'y':cos})
            g_orbit.put({'x':cos,'y':sin})
            #辞書型をファイルインスタンスの'write_data'メソッドに渡すことでファイルにデータを書き出す。キーと同じヘッダを持つ列に値が書き込まれる。
            #Write data to the file by handing a dict to 'write_data' method of the file instance.
            #A value is written to the column which has the same header with the key.
            data={T:t,Sin:sin,Cos:cos}
            file.write_data(data)
            time.sleep(0.01)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    s=Sample()
    sys.exit(app.exec_())       


# -*- coding: utf-8 -*-
"""
Created on Tue Dec 12 12:36:24 2017

@author: Yuki
"""

'''
The widgets for the configure of Kuchinawa in the setting window 
made by QtDesigner have names starting with 'config_'.
'''

from kuchinawa import Setting,Configure
from PyQt5.QtWidgets import QComboBox,QSpinBox
from distutils.util import strtobool

class Configure_interface():
    PREFIX='config_'
    
    def __init__(self,mydict):
        '''
        Make the interface to interact with the configure window
        
        Parameters:
            mydict: dict
                dictionary whose key and value are widget name and widget respectively
        '''
        self.configures={}
        for key,widget in mydict.items():
            if key.startswith(Configure_interface.PREFIX):
                self.configures[key]=widget
    
    def set_value(self,key,value):
        '''
        Set value to a widget
        
        Parameters:
            key: str
                the widget's name
            value: bool,int,float,etc.
                the value to be set
        '''
        widget=self.configures[key]
        if isinstance(widget,QComboBox):#index 0: True, index1: False
            if value:
                widget.setCurrentIndex(0)
            else:
                widget.setCurrentIndex(1)
        elif isinstance(widget,QSpinBox):
            widget.setValue(value)
            
    def get_value(self,key):
        '''
        Return the value of a widget
        
        Parameters:
            key: str
                the widget's name
        Returns:
            bool,int,float,etc.
        '''
        widget=self.configures[key]
        if isinstance(widget,QComboBox):#index 0: True, index1: False
            return bool(strtobool(widget.currentText()))
        elif isinstance(widget,QSpinBox):
            return widget.value()
        
    def set_all_values(self,mydict):
        '''
        Set all the values to the widgets at once
        
        Parameters:
            mydict: dict
                dictionary whose key and value are widget name and widget respectively
        '''
        for key,value in mydict.items():
            self.set_value(key,value)
            
    def get_all_values(self):
        '''
        Get all the values from the widgets at once
        
        Returns:
            dict
        '''
        answer={}
        for key in self.configures.keys():
            answer[key]=self.get_value(key)
        return answer
                

class My_Ui_Form(Setting.Ui_Form):
    def make_interface(self):
        self._interface=Configure_interface(vars(self))
        self.pushButton_ok.pressed.connect(self.ok_pressed)
        self.pushButton_cancel.pressed.connect(self.cancel_pressed)
        
    def ok_pressed(self):
        window_config=self._interface.get_all_values()
        Configure.Configure.setAllConfigure(window_config)
        
    def cancel_pressed(self):
        current=Configure.Configure.getConfigure()
        self._interface.set_all_values(current)
        


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\Yuki\Dropbox\python\my_module_root\kuchinawa\kuchinawa\Examples\Thread.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(603, 648)
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setGeometry(QtCore.QRect(40, 40, 481, 141))
        self.groupBox.setObjectName("groupBox")
        self.lcd1 = QtWidgets.QLCDNumber(self.groupBox)
        self.lcd1.setGeometry(QtCore.QRect(10, 40, 241, 61))
        self.lcd1.setDigitCount(7)
        self.lcd1.setObjectName("lcd1")
        self.button1 = QtWidgets.QPushButton(self.groupBox)
        self.button1.setGeometry(QtCore.QRect(270, 20, 93, 28))
        self.button1.setObjectName("button1")
        self.bar1 = QtWidgets.QProgressBar(self.groupBox)
        self.bar1.setGeometry(QtCore.QRect(300, 90, 118, 23))
        self.bar1.setProperty("value", 0)
        self.bar1.setObjectName("bar1")
        self.groupBox_2 = QtWidgets.QGroupBox(Form)
        self.groupBox_2.setGeometry(QtCore.QRect(40, 220, 481, 141))
        self.groupBox_2.setObjectName("groupBox_2")
        self.lcd2 = QtWidgets.QLCDNumber(self.groupBox_2)
        self.lcd2.setGeometry(QtCore.QRect(10, 40, 241, 61))
        self.lcd2.setDigitCount(7)
        self.lcd2.setObjectName("lcd2")
        self.button2 = QtWidgets.QPushButton(self.groupBox_2)
        self.button2.setGeometry(QtCore.QRect(270, 20, 93, 28))
        self.button2.setObjectName("button2")
        self.bar2 = QtWidgets.QProgressBar(self.groupBox_2)
        self.bar2.setGeometry(QtCore.QRect(300, 90, 118, 23))
        self.bar2.setProperty("value", 0)
        self.bar2.setObjectName("bar2")
        self.groupBox_3 = QtWidgets.QGroupBox(Form)
        self.groupBox_3.setGeometry(QtCore.QRect(40, 410, 481, 141))
        self.groupBox_3.setObjectName("groupBox_3")
        self.lcd3 = QtWidgets.QLCDNumber(self.groupBox_3)
        self.lcd3.setGeometry(QtCore.QRect(10, 40, 241, 61))
        self.lcd3.setDigitCount(7)
        self.lcd3.setObjectName("lcd3")
        self.button3 = QtWidgets.QPushButton(self.groupBox_3)
        self.button3.setGeometry(QtCore.QRect(270, 20, 93, 28))
        self.button3.setObjectName("button3")
        self.bar3 = QtWidgets.QProgressBar(self.groupBox_3)
        self.bar3.setGeometry(QtCore.QRect(300, 90, 118, 23))
        self.bar3.setProperty("value", 0)
        self.bar3.setObjectName("bar3")
        self.groupBox.raise_()
        self.groupBox_2.raise_()
        self.groupBox_2.raise_()
        self.groupBox_3.raise_()

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.groupBox.setTitle(_translate("Form", "Thread1"))
        self.button1.setText(_translate("Form", "terminate"))
        self.groupBox_2.setTitle(_translate("Form", "Thread2"))
        self.button2.setText(_translate("Form", "terminate"))
        self.groupBox_3.setTitle(_translate("Form", "Thread3"))
        self.button3.setText(_translate("Form", "terminate"))




# -*- coding: utf-8 -*-
import multiprocessing
import sys,time
from datetime import datetime

from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QColor
from PyQt5.QtCore import QThread

import numpy as np

from kuchinawa.Thread import Main
from kuchinawa import Graph

import Thread_ui

class MyThread(QThread):
    def __init__(self,call,lcd,progress,stop_signal,graph):
        super().__init__()
        self.call=call
        self.lcd=lcd
        self.progress=progress
        stop_signal.connect(self.stop)
        self.graph=graph
        self.run=True
        
    def run(self):
        timeOrigin=datetime.now() #get the origin of time
        self.run=True
        while self.run:
            time.sleep(0.01)
            t=(datetime.now()-timeOrigin).total_seconds()
            r=np.random.rand()
            self.call(self.lcd.display,(r,))
            self.graph.put({'x':t,'y':r})
        
    def stop(self):
        self.run=False
        print('stop')
        
T='Time'
V='Value'
COLORS=[QColor(255,0,0),QColor(255,255,0),QColor(0,255,0)] #the colors of points in graphs

class Sample(Main):
    def __init__(self):
        super().__init__()
        self.ui=Thread_ui.Ui_Form()
        self.setUI(self.ui)
        self.g1=self.addGraph(Graph.ScatterAll,(T,'sec',V,'V'),{'color':COLORS[0]},'Thread1')
        self.g2=self.addGraph(Graph.ScatterAll,(T,'sec',V,'V'),{'color':COLORS[1]},'Thread2')
        self.g3=self.addGraph(Graph.ScatterAll,(T,'sec',V,'V'),{'color':COLORS[2]},'Thread3')
        self.t1=MyThread(self.call,self.ui.lcd1,self.ui.bar1,self.ui.button1.pressed,self.g1)
        self.t2=MyThread(self.call,self.ui.lcd2,self.ui.bar2,self.ui.button2.pressed,self.g2)
        self.t3=MyThread(self.call,self.ui.lcd3,self.ui.bar3,self.ui.button3.pressed,self.g3)
        
    def run(self):
        self.t1.start()
        self.t2.start()
        self.t3.start()
        
        self.t1.wait()
        self.t2.wait()
        self.t3.wait()
        print('hoge')

if __name__ == '__main__':
    app = QApplication(sys.argv)
    s=Sample()
    sys.exit(app.exec_())       


# -*- coding: utf-8 -*-
"""
Created on Sat Sep  2 16:00:00 2017

@author: Yuki
"""

from PyQt5 import uic
import sys,os,shutil
import kuchinawa.Template

def compileUi(path):
    '''
    Compile a .ui file made by QtDesigner to a .py file.
    
    Paramters:
        path: str
            the path to the .ui file
    Returns:
        str
            the path to the created .py file
    '''
    def checkLines(target,lines):
        for index,line in enumerate(lines):
            if target in line:
                return index
    
    other,ext=os.path.splitext(path)
    if not ext=='.ui':
        raise Exception('Input file must be a file with the extension .ui')
    else:
        with open(path,'r') as i:
            o_path=other+'_ui.py'
            with open(o_path,'w') as o:
                uic.compileUi(i,o)
                print('{:*^50}'.format('compiled'))
                print('Source: {0}'.format(path))
                print('Output: {0}'.format(o_path))
        pro_path=other+'.py'
        if os.path.isfile(pro_path):
            pass
        else:
            #create a template file
            temp_path=kuchinawa.Template.__file__
            with open(temp_path,'r') as temp:
                lines=temp.readlines()
            
            index=checkLines('class Sample(Main):',lines)
            directory,module=os.path.split(other)
            module_name=module+'_ui'
            import_str='import '+module_name+'\n\n'
            lines.insert(index,import_str)
            
            index=checkLines('super().__init__()',lines)
            super_str=lines[index]
            indent=super_str.split('super()')[0]
            instantiate_str=indent+'self.ui='+module_name+'.Ui_Form()\n'
            set_str=indent+'self.setUI(self.ui)\n'
            lines.insert(index+1,instantiate_str)
            lines.insert(index+2,set_str)
            
            with open(pro_path,'w') as program:
                program.writelines(lines)
                print('{:*^50}'.format('generated a template file'))
                print('Template: {0}'.format(pro_path))


# -*- coding: utf-8 -*-
"""
Created on Mon May 15 21:26:18 2017

@author: Press150
"""

import visa
import re

class GpibInst():
    pattern=re.compile(r'^[\+-]?\d*\.?\d*E[\+-]?\d*')
    def __init__(self,gpib):
        rm=visa.ResourceManager()
        self.inst = rm.get_instrument("GPIB::"+ str(int(gpib)))
        
    def query(self,command,raw_string=False):
        answer=self.inst.query(command)
        if raw_string:
            return answer
        else:
            body=self.pattern.match(answer).group()
            return float(body)
    
    def write(self,command):
        self.inst.write(command)
        return True


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'setting.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(351, 331)
        self.tabWidget = QtWidgets.QTabWidget(Form)
        self.tabWidget.setGeometry(QtCore.QRect(30, 20, 291, 231))
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayoutWidget = QtWidgets.QWidget(self.tab)
        self.gridLayoutWidget.setGeometry(QtCore.QRect(20, 30, 241, 131))
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.gridLayoutWidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.config_graph_clear = QtWidgets.QComboBox(self.gridLayoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.config_graph_clear.sizePolicy().hasHeightForWidth())
        self.config_graph_clear.setSizePolicy(sizePolicy)
        self.config_graph_clear.setEditable(False)
        self.config_graph_clear.setMaxVisibleItems(2)
        self.config_graph_clear.setObjectName("config_graph_clear")
        self.config_graph_clear.addItem("")
        self.config_graph_clear.addItem("")
        self.gridLayout.addWidget(self.config_graph_clear, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.gridLayoutWidget)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.config_graph_rate = QtWidgets.QSpinBox(self.gridLayoutWidget)
        self.config_graph_rate.setMinimum(100)
        self.config_graph_rate.setMaximum(10000)
        self.config_graph_rate.setSingleStep(100)
        self.config_graph_rate.setObjectName("config_graph_rate")
        self.gridLayout.addWidget(self.config_graph_rate, 1, 1, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.tabWidget.addTab(self.tab_2, "")
        self.pushButton_ok = QtWidgets.QPushButton(Form)
        self.pushButton_ok.setGeometry(QtCore.QRect(120, 280, 81, 28))
        self.pushButton_ok.setObjectName("pushButton_ok")
        self.pushButton_cancel = QtWidgets.QPushButton(Form)
        self.pushButton_cancel.setGeometry(QtCore.QRect(220, 280, 93, 28))
        self.pushButton_cancel.setObjectName("pushButton_cancel")

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(0)
        self.config_graph_clear.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label.setText(_translate("Form", "clear when restarting"))
        self.config_graph_clear.setCurrentText(_translate("Form", "True"))
        self.config_graph_clear.setItemText(0, _translate("Form", "True"))
        self.config_graph_clear.setItemText(1, _translate("Form", "False"))
        self.label_2.setText(_translate("Form", "update interval (/ms)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("Form", "Graph"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("Form", "Blank"))
        self.pushButton_ok.setText(_translate("Form", "Apply"))
        self.pushButton_cancel.setText(_translate("Form", "Cancel"))




# -*- coding: utf-8 -*-
import sys,time
from datetime import datetime

from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QColor

import numpy as np

from kuchinawa.Thread import Main
from kuchinawa import Graph

class Sample(Main):
    def __init__(self):
        super().__init__()
        
    def run(self):
        #Initialization
        
        while self.running==True:
            #Loop process
            pass

if __name__ == '__main__':
    app = QApplication(sys.argv)
    s=Sample()
    sys.exit(app.exec_())       


# -*- coding: utf-8 -*-
"""
Created on Mon Dec  4 00:58:12 2017

@author: Yuki
"""
import kuchinawa
from kuchinawa import Logger
import os,copy,pickle

class Configure:
    _CONFIG_PATH=os.path.join(kuchinawa.ENTRYPOINT,'config.dump')
    _DEFAULT_CONFIG={'config_graph_clear':True,'config_graph_rate':100}
    _config=copy.copy(_DEFAULT_CONFIG)
    
    @classmethod
    def loadConfigure(cls,path=_CONFIG_PATH):
        '''
        Load the configure file
        
        Parameters:
            path: str
                the path to the configure file
        '''
        try:
            with open(path,'rb') as config:
                cls._config=pickle.load(config)
                Logger.logger.debug('Loaded the configure file from {}'.format(path))
        except:
            Logger.logger.debug('Failed loading the configure file from {}'.format(path))
    
    @classmethod
    def saveConfigure(cls,path=_CONFIG_PATH):
        '''
        Save the configure to the file
        
        Parameters:
            path: str
                the path to the configure file
        '''
        with open(path,'wb') as config:
            pickle.dump(cls._config,config)
            Logger.logger.debug('Saved the configure file to {}'.format(path))
    
    @classmethod
    def resetConfigure(cls):
        '''
        Reset the configure
        '''
        cls._config=copy.copy(cls._DEFAULT_CONFIG)
        Logger.logger.debug('Reset the configure')
        
    @classmethod
    def getConfigure(cls):
        '''
        Get the configure
        
        Returns:
            dict
                the current configure
        '''
        return copy.copy(cls._config)
    
    @classmethod
    def setEachConfigure(cls,key,value):
        '''
        Set each of the configure
        
        Parameters:
            key: str
                the key of the configure to change
            value: object
                the value of the configure to be set
        '''
        cls._config[key]=value
    
    @classmethod
    def setAllConfigure(cls,mydict):
        '''
        Set all the configure at once
        
        Parameters:
            mydict: dict
                the configure to be set
        '''
        cls._config=copy.copy(mydict)
    


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\Yuki\Dropbox\python\my_module_root\kuchinawa\kuchinawa\Exapmles\SinCos.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(928, 669)
        self.groupBox = QtWidgets.QGroupBox(Form)
        self.groupBox.setGeometry(QtCore.QRect(10, 200, 451, 301))
        self.groupBox.setObjectName("groupBox")
        self.lcdNumber_sin = QtWidgets.QLCDNumber(self.groupBox)
        self.lcdNumber_sin.setGeometry(QtCore.QRect(40, 60, 191, 61))
        self.lcdNumber_sin.setObjectName("lcdNumber_sin")
        self.dial_sin = QtWidgets.QDial(self.groupBox)
        self.dial_sin.setGeometry(QtCore.QRect(40, 130, 191, 171))
        self.dial_sin.setObjectName("dial_sin")
        self.amp_sin = QtWidgets.QSlider(self.groupBox)
        self.amp_sin.setGeometry(QtCore.QRect(290, 80, 22, 160))
        self.amp_sin.setProperty("value", 50)
        self.amp_sin.setOrientation(QtCore.Qt.Vertical)
        self.amp_sin.setObjectName("amp_sin")
        self.noise_sin = QtWidgets.QSlider(self.groupBox)
        self.noise_sin.setGeometry(QtCore.QRect(370, 80, 22, 160))
        self.noise_sin.setOrientation(QtCore.Qt.Vertical)
        self.noise_sin.setObjectName("noise_sin")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setGeometry(QtCore.QRect(260, 40, 62, 15))
        self.label.setObjectName("label")
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setGeometry(QtCore.QRect(350, 40, 62, 15))
        self.label_2.setObjectName("label_2")
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setGeometry(QtCore.QRect(40, 30, 62, 15))
        self.label_5.setObjectName("label_5")
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setGeometry(QtCore.QRect(40, 140, 62, 15))
        self.label_6.setObjectName("label_6")
        self.groupBox_3 = QtWidgets.QGroupBox(Form)
        self.groupBox_3.setGeometry(QtCore.QRect(470, 200, 451, 301))
        self.groupBox_3.setObjectName("groupBox_3")
        self.lcdNumber_cos = QtWidgets.QLCDNumber(self.groupBox_3)
        self.lcdNumber_cos.setGeometry(QtCore.QRect(40, 60, 191, 61))
        self.lcdNumber_cos.setObjectName("lcdNumber_cos")
        self.dial_cos = QtWidgets.QDial(self.groupBox_3)
        self.dial_cos.setGeometry(QtCore.QRect(40, 130, 191, 171))
        self.dial_cos.setObjectName("dial_cos")
        self.amp_cos = QtWidgets.QSlider(self.groupBox_3)
        self.amp_cos.setGeometry(QtCore.QRect(290, 80, 22, 160))
        self.amp_cos.setProperty("value", 50)
        self.amp_cos.setOrientation(QtCore.Qt.Vertical)
        self.amp_cos.setObjectName("amp_cos")
        self.noise_cos = QtWidgets.QSlider(self.groupBox_3)
        self.noise_cos.setGeometry(QtCore.QRect(370, 80, 22, 160))
        self.noise_cos.setOrientation(QtCore.Qt.Vertical)
        self.noise_cos.setObjectName("noise_cos")
        self.label_13 = QtWidgets.QLabel(self.groupBox_3)
        self.label_13.setGeometry(QtCore.QRect(260, 40, 62, 15))
        self.label_13.setObjectName("label_13")
        self.label_14 = QtWidgets.QLabel(self.groupBox_3)
        self.label_14.setGeometry(QtCore.QRect(350, 40, 62, 15))
        self.label_14.setObjectName("label_14")
        self.label_15 = QtWidgets.QLabel(self.groupBox_3)
        self.label_15.setGeometry(QtCore.QRect(40, 30, 62, 15))
        self.label_15.setObjectName("label_15")
        self.label_16 = QtWidgets.QLabel(self.groupBox_3)
        self.label_16.setGeometry(QtCore.QRect(40, 140, 62, 15))
        self.label_16.setObjectName("label_16")
        self.lineEdit = QtWidgets.QLineEdit(Form)
        self.lineEdit.setGeometry(QtCore.QRect(90, 90, 581, 21))
        self.lineEdit.setObjectName("lineEdit")
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setGeometry(QtCore.QRect(80, 40, 171, 31))
        self.label_3.setStyleSheet("font: 12pt \"MS UI Gothic\";")
        self.label_3.setObjectName("label_3")

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.groupBox.setTitle(_translate("Form", "Sin"))
        self.label.setText(_translate("Form", "Amp"))
        self.label_2.setText(_translate("Form", "Noise"))
        self.label_5.setText(_translate("Form", "Current"))
        self.label_6.setText(_translate("Form", "Phase"))
        self.groupBox_3.setTitle(_translate("Form", "Cos"))
        self.label_13.setText(_translate("Form", "Amp"))
        self.label_14.setText(_translate("Form", "Noise"))
        self.label_15.setText(_translate("Form", "Current"))
        self.label_16.setText(_translate("Form", "Phase"))
        self.label_3.setText(_translate("Form", "Output file path: "))




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 10 00:25:28 2017

@author: Yuki
"""

import logging
logger = logging.getLogger('Kuchinawa Log')
 
# ログレベルの設定
logger.setLevel(10)
 
# ログのファイル出力先を設定
fh = logging.FileHandler('kuchinawa.log')
logger.addHandler(fh)
 
# ログのコンソール出力の設定
sh = logging.StreamHandler()
logger.addHandler(sh)
 
# ログの出力形式の設定
formatter = logging.Formatter('%(asctime)s:%(lineno)d:%(levelname)s:%(message)s')
fh.setFormatter(formatter)
sh.setFormatter(formatter)


# -*- coding: utf-8 -*-
"""
Created on Mon Oct 23 03:24:37 2017

@author: Yuki
"""
import os,sys,logging

ENTRYPOINT=__path__[0]
ICONPATH=os.path.join(ENTRYPOINT,'Icons','logo.png')
KUCHINAWA='Kuchinawa'

from kuchinawa.Compile import compileUi
from kuchinawa.Thread import Main

#change the multiprocessing's context to 'spawn'
try:
    import multiprocessing
    multiprocessing.set_start_method('spawn')
except:
    print('The context of multiprocessing is already set.')

def run_sample():
    '''Run a sample program'''
    from PyQt5.QtWidgets import QApplication
    from kuchinawa.Examples import SinCos
    app = QApplication([])
    s=SinCos.Sample()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""
Created on Sat Sep  2 00:56:52 2017

@author: Yuki
"""
import sys

from multiprocessing import Queue,Process
from abc import abstractmethod

from PyQt5.QtCore import pyqtSignal,QThread
from PyQt5.QtWidgets import QToolBar,QAction,QMainWindow,QWidget,QApplication,\
                            QScrollArea,QFileDialog,QDesktopWidget
from PyQt5.QtGui import QIcon                        

from kuchinawa import Graph,File,Setting_extend
import kuchinawa

class MyToolBar(QToolBar):
    '''
    A toolbar which has several states. Each state has specific executable actions.
    '''
    READY=0     #a flag which means the program is ready to start
    RUNNING=1   #the program is running
    
    def __init__(self):
        super().__init__()
        self.exeAction=QAction('Run', self)
        self.stopAction = QAction('Stop', self)
        self.debugAction = QAction('Debug', self)
        self.settingAction=QAction('Settings',self)
        self.exitAction=QAction('Exit',self)
        self.addAction(self.exeAction)
        self.addAction(self.stopAction)
        self.addAction(self.debugAction)
        self.addAction(self.settingAction)
        self.addAction(self.exitAction)
        
    def setState(self,end):
        '''change the current state'''
        if end==MyToolBar.READY:
            self.exeAction.setEnabled(True)
            self.stopAction.setEnabled(False)
            self.debugAction.setEnabled(True)
            self.settingAction.setEnabled(True)
            self.exitAction.setEnabled(True)
        elif end==MyToolBar.RUNNING:
            self.exeAction.setEnabled(False)
            self.stopAction.setEnabled(True)
            self.debugAction.setEnabled(False)
            self.settingAction.setEnabled(True)
            self.exitAction.setEnabled(True)
            

class Main(QThread):
    '''
    A base class for measurement programs. Users can make a measurement program by inheriting this class.
    '''
    
    __callSignal=pyqtSignal(object,tuple,dict)
    
    def __init__(self):
        super().__init__()
        self.__initUI()
        self.finished.connect(self.__finalize)
        self.running=False #a flag which determins whether run method ends or not
        
        self.que=Queue() #connect between main and graph drawing process
        self.p=Process(target=Graph.initGraphContainer,args=(self.que,))
        self.p.start()
        
        
        self.result=Queue() #queue to put a result of a function called in the thread in which itself exists
        self.__callSignal.connect(self.__call)
        
        #place the window to the left of the screen
        screen=QDesktopWidget().availableGeometry()
        height=screen.height()
        width=screen.width()/2
        self.mw.setGeometry(0,40,width,height)
        self.mw.setWindowIcon(QIcon(kuchinawa.ICONPATH))
        self.mw.setWindowTitle(kuchinawa.KUCHINAWA+' Control Panel')
        self.mw.show()
        
    
    def __initUI(self):
        self.mw=QMainWindow()
        self.scroll=QScrollArea()
        self.mw.setCentralWidget(self.scroll)
        
        self.toolbar=MyToolBar()
        self.toolbar.exeAction.triggered.connect(self.__exePressed)
        self.toolbar.stopAction.triggered.connect(self.__stopPressed)
        self.toolbar.exitAction.triggered.connect(self.__exitPressed)
        self.toolbar.settingAction.triggered.connect(self.__settingPressed)
        self.toolbar.setState(MyToolBar.READY)
        self.mw.addToolBar(self.toolbar)
        
    def setUI(self,ui):
        '''
        Set a user interface made by qtdesigner to the MainWindow.
        
        Parameters:
            ui: UI_Form
            
        Returns:
            None
        
        '''
        self.widget=QWidget()
        ui.setupUi(self.widget)
        self.scroll.setWidget(self.widget)
        
    @abstractmethod    
    def run(self):
        '''
        The body of a measurement program. Users implement this method for their measurement.
        '''
        raise NotImplementedError('This method must be implemented in a subclass.')
            
    def __exePressed(self):
        self.toolbar.setState(MyToolBar.RUNNING)
        self.__resetGraphs()
        self.running=True
        self.start()
        
    def __exitPressed(self):
        self.__stopPressed()
        self.que.put(Graph.PoisonPill())
        del(self.mw)
        
    def __settingPressed(self):
        try:
            self.settingWindow.showNormal()
        except:
            self.settingWindow=QWidget()
            self.settingUi=Setting_extend.My_Ui_Form()
            self.settingUi.setupUi(self.settingWindow)
            self.settingUi.make_interface()
            self.settingWindow.show()
        
    def __stopPressed(self):
        self.running=False
        
    def __finalize(self):
        self.toolbar.setState(MyToolBar.READY)
        self.running=False
        
    def call(self,function,args=None,kwargs=None):
        '''
        Call a function in the thread in which the instance of this class exists.
        
        Paramters:
            function: function
                a function to call
            args: tuple
                positional arguments
            kwargs: dict
                keyward arguments
                
        Returns:
            object
                depends on the function
        '''
        if args==None:
            args=()
        if kwargs==None:
            kwargs={}
        self.__callSignal.emit(function,args,kwargs)
        result=self.result.get()
        return result
        
    def __call(self,function,args=None,kwargs=None):
        '''
        A slot method for __callSignal. This method is executed when __callSignal is emit and put the return to self.result.
        '''
        if args==None:
            args=()
        if kwargs==None:
            kwargs={}
        result=function(*args,**kwargs)
        self.result.put(result)
        
    def addGraph(self,graph,args=None,kwargs=None,name='Unnamed'):
        '''
        Add a graph to GraphContainer.
        
        Parameters:
            graph: class
                a subclass of Graph.GraphBase
            *args: tuple
                positional arguments to pass to __init__ of 'graph'
            **kwargs: dict
                keyword arguments to pass to __init__ of 'graph'
            name: string
                the window title of the graph
        Returns: Graph.GraphInterface
            the interface to plot data to the graph
        '''
        if args==None:
            args=()
        if kwargs==None:
            kwargs={}
        interface=Graph.GraphInterface(self.que,graph,name,*args,**kwargs)
        return interface
        
    def __resetGraphs(self):
        '''
        Reset GraphContainer.
        
        Parameters:
            None
        Returns:
            None
        '''
        self.que.put(Graph.ResetSignal())
        
    def getSaveFile(self,sep='\t',comment='#'):
        '''
        Open a file dialog and return a csv file instance holding the path appointed in the dialog.
        
        Parameters:
            sep: str
                the seperater between values in the csv file
            comment: str
                the header of comment lines
        Returns:
            File.File
        '''
        def fileDialog(parent):
            path,hoge = QFileDialog.getSaveFileName(parent, 'select a file to write data to')
            return path
        path=self.call(fileDialog,(self.mw,))
        file=File.File(path)
        return file

        
if __name__ == '__main__':
    app = QApplication(sys.argv)
    t=Main()
    sys.exit(app.exec_())


# -*- coding: utf-8 -*-
"""
Created on Tue Sep  5 11:33:04 2017

@author: Jackson
"""
import sys
import pandas as pd

class File():
    '''
    Implements a csv type file instance.
    '''
    
    def __init__(self,path,sep='\t',comment='#'):
        super().__init__()
        self.path=path
        self.file=open(path,'a+')
        self.sep=sep
        self.comment=comment
        self._before_writing=True
        
    def close(self):
        self.file.close()
        
    def save(self):
        self.close()
        self.file=open(self.path,'a+')
        
    def write_comment(self,string):
        '''
        Add a comment to the head of the file
        
        Parameters
            string: string
                A string to be added. \n is not needed.
                
        Returns
            None
        '''
        if self._before_writing:
            self.file.write(self.comment+string+'\n')
            self.save()
        else:
            raise Exception('Comments must be written before starting to write data')
    
    def write_header(self,mylist):
        '''
        Set headers to the file.
        
        Parameters
            mylist: list
                The header names.
                
        Returns
            None
        '''
        if self._before_writing:
            self.columns=pd.DataFrame(columns=mylist)
            self.columns.to_csv(self.file,sep=self.sep,index=False)
            self._before_writing=False
            self.save()
        else:
            raise Exception('Headers are already written')
    
    def write_data(self,mydict):
        '''
        Write data to the file
        
        Parameters
            mydict: dictionary
                The keys corresponds to the headers.
                
        Returns
            None
        '''
        if self._before_writing:
            raise Exception('Headers has not been set yet.')
        else:
            row=self.columns.append(mydict,ignore_index=True)
            row.to_csv(self.file,sep=self.sep,index=False,header=False)
            self.save()


# coding: utf-8

try:
    import setuptools
    from setuptools import setup, find_packages
except ImportError:
    print("Please install setuptools.")

import os
long_description = 'This package help you to create a control program for measurement instruments. It provides you a dynamic way to control your program via a graphical user interface made by QtDesigner. Besides, it provides you a function of realtime graph plotting.'

setup(
    name  = 'kuchinawa',
    version = '0.4',
    description = 'Development environment to create a control program for measurement instruments',
    long_description = long_description,
    license = 'MIT',
    author = 'Yuki Arai',
    author_email = 'threemeninaboat3247@gmail.com',
    url = 'https://github.com/threemeninaboat3247/kuchinawa',
    keywords = 'measurement',
    packages = find_packages(),
    package_data={
        # If any package contains *.txt or *.rst files, include them:
        'kuchinawa': ['Icons/*.png']
    },
    install_requires = ['pyqtgraph>=0.10.0','pandas','pyvisa'],
    classifiers = [
      'Programming Language :: Python :: 3',
      'Intended Audience :: Science/Research',
      'License :: OSI Approved :: MIT License'
    ]
)
